/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 2007 Sun Microsystems, Inc. All rights reserved. 
 * 
 * The contents of this file are subject to the terms of the Common Development
 * and Distribution License("CDDL") (the "License").  You may not use this file
 * except in compliance with the License. 
 * 
 * You can obtain a copy of the License at:
 *     https://jersey.dev.java.net/license.txt
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * When distributing the Covered Code, include this CDDL Header Notice in each
 * file and include the License file at:
 *     https://jersey.dev.java.net/license.txt
 * If applicable, add the following below this CDDL Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 *     "Portions Copyrighted [year] [name of copyright owner]"
 */

package com.sun.ws.rest.impl.model;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

/**
 *
 * @author Paul.Sandoz@Sun.Com
 */
class MethodList implements Iterable<Method> {

    Method[] methods;
    
    MethodList(Class c) {
        this(c.getMethods());
    }
    
    MethodList(Method... methods) {
        this.methods = methods;
    }

    public Iterator<Method> iterator() {
      return Arrays.asList(methods).iterator();
    }
    
    <T extends Annotation> MethodList hasAnnotation(final Class<T> annotation) {
        return filter(new Filter() {
            public boolean keep(Method m) {
                return m.getAnnotation(annotation) != null;
            }
        });
    }
    
    <T extends Annotation> MethodList hasNotAnnotation(final Class<T> annotation) {
        return filter(new Filter() {
            public boolean keep(Method m) {
                return m.getAnnotation(annotation) == null;
            }
        });
    }
    
    private interface Filter {
        boolean keep(Method m);
    }
    
    private MethodList filter(Filter f) {
        List<Method> r = new ArrayList<Method>();
        for (Method m : methods)
            if (f.keep(m))
                r.add(m);
        return new MethodList(r.toArray(new Method[0]));
    }
}